;
;****************************************************************************
;
;  Purpose:
;	Interrupt Driven Serial Port Handler, Internal Memory Version
;
;  Date:
;	05/16/90
;
;  Author:
;	John C. Wren
;
;  Modications:
;	02/06/97 - Added Description Fields For Archive
;
;  Processor:
;	Dallas 80C320 And Variants With Second Serial Port
;
;  Assembler:
;	Avocet AVA51
;
;  Dependencies:
;	Timer 0 Or 2 Setup For Baud Rate Generation
;
;  Files:
;	None
;
;  Comments:
;	A LJMP To SP1_INT Needs To Be Placed At Location 003bh.  Since The
;	Input And Output Buffers Are Referenced Only Indirectly, They Can
;	Be Located To The Indirect (080h..0ffh) Memory On Processors With
;	More Than 128 Bytes Of Internal RAM.
;
;  Philosophic:
;	The Equates In The Equates Section Should Be In The System 
;	EQUATES.INC File.
;
;****************************************************************************
;
;  Includes
;
;		%include "equates.inc"
		seg	code
;
;****************************************************************************
;
;  Publics
;
		public	SP1_INIT   
		public	SP1_INT    
		public	SP1_MCT
		public	SP1_MDT
		public	SP1_IN	   
		public	SP1_INS    
		public	SP1_INCLEAR
		public	SP1_OUT    
		public	SP1_OUTS
		public	SP1_OUTEMPTY
;
;****************************************************************************
;
;  Equates
;
ETX		equ	3			; ASCII ETX Character
;
;  These Are The Buffer Sizes, And Can Be Up To 255 Bytes
;
E_SP1IBL	equ	16			; 32 Character Input Buffer
E_SP1OBL	equ	16			; 16 Character Output Buffer
;
;*******************************************************************************
;
;  Define Register Unique To The Dallas 80C320/80C520
;
dpl1		equ	084h			; 320/520
dph1		equ	085h			; 320/520
dps		equ	086h			; 320/520
ckcon		equ	08eh			; 320/520
exif		equ	091h			; 320/520
scon1		equ	0c0h			; 320/520
sbuf1		equ	0c1h			; 320/520
romsize		equ	0c2h			; 520 Only
pmr		equ	0c4h			; 520 Only
status		equ	0c5h			; 320/520
ta		equ	0c7h			; 320/520
wdcon		equ	0d8h			; 320/520
eie		equ	0e8h			; 320/520
eip		equ	0f8h			; 320/520
;
ri1		bit	scon1.0
ti1		bit	scon1.1
rb81		bit	scon1.2
tb81		bit	scon1.3
ren1		bit	scon1.4
sm21		bit	scon1.5
sm11		bit	scon1.6
sm01		bit	scon1.7
es1		bit	ie.6
ps1		bit	ip.6
;
;****************************************************************************
;
;  Description:
;	Initialize Serial Port 0 Variables, Serial Port
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	None
;
;  Affected:
;	PSW.P, PSW.Z, Acc
;
;  Stack:
;	0 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	Sets Serial Port To Mode 1.  Timer 1 Or 2 (If Applicable) Should 
;	Already Be Setup In 8-Bit Auto Reload Mode For Baud Rate Generation.  
;	Must Be Called Before Any Serial I/O Performed.
;
;
;****************************************************************************
;
; SP1_INIT - Init SP1 Routines (Does Nothing, Currently)
;
SP1_INIT	proc
		clr	a			; 0
		mov	D_SP1IBH,a		; Set Input Buffer Header To 0
		mov	D_SP1IBT,a		; Set Input Buffer Tail To 0
		mov	D_SP1ICC,a		; Set Input Character Count To 0
		mov	D_SP1OBH,a		; Set Output Buffer Header To 0
		mov	D_SP1OBT,a		; Set Output Buffer Tail To 0
		mov	D_SP1OCC,a		; Set Output Character Count To 0
		clr	B_SP1OFL		; Set No Characters To Go Out
;
		mov	scon1,#040h		; Set Serial Mode 1
		clr	ri1			; Clear Receive Interrupt Flag
		setb	ti1			; Pretend Character Sent OK
		setb	ren1			; Allow Characters To Be Received
		setb	ps1			; Set Serial As High Priority
		setb	es1			; Allow Interrupts In Interrupt Structure
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Output String From Code Area
;
;  Entry Requirements:
;	DPTR Points To ETX Terminated String In Code Space
;
;  On Exit:
;	None
;
;  Affected:
;	PSW.CY
;
;  Stack:
;	5 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
SP1_MCT 	proc
		push	dpl			; Save DPL
		push	dph			; Save DPH
		push	acc			; Save Acc
l?p1		clr	a			; Make A = 0 So MOVC Works
		movc	a,@a+dptr		; Get Character
		inc	dptr			; Point To Next Character
		cjne	a,#ETX,l?p2		; If Not EOM, Output Character
		pop	acc			; Recover Acc
		pop	dph			; Recover DPH
		pop	dpl			; Recover DPL
		ret				; Return To Caller
l?p2		call	SP1_OUT 		; Output Character
		sjmp	l?p1			; And Loop
		endproc
;
;****************************************************************************
;
;  Description:
;	Output String From External Memory Area
;
;  Entry Requirements:
;	DPTR Points To ETX Terminated String In External RAM 
;
;  On Exit:
;	None
;
;  Affected:
;	PSW.CY
;
;  Stack:
;	5 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
SP1_MDT 	proc
		push	dpl			; Save DPL
		push	dph			; Save DPH
		push	acc			; Save Acc
l?p1		movx	a,@dptr 		; Get Character
		inc	dptr			; Point To Next Character
		cjne	a,#ETX,l?p2		; If Not EOM, Output Character
		pop	acc			; Recover Acc
		pop	dph			; Recover DPH
		pop	dpl			; Recover DPL
		ret				; Return To Caller
l?p2		call	SP1_OUT 		; Output Character
		sjmp	l?p1			; And Loop
		endproc
;
;****************************************************************************
;
;  Description:
;	Interrupt Source Determiner
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	None
;
;  Affected:
;	None
;
;  Stack:
;	0 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	Normally, The RI Or TI Jump Should Always Be Taken.  On One 
;	Particular Emulator (That I'd Prefer Not To Mention Here), We've 
;	Seen It Generate A Serial Interrupt, With Neither RI Or TI Set.  
;	It Seems To Be Related To Transmitting, And I've Found That It
;	Works Best To Clear The Flag That Tells The Serial Transmit
;	Interrupt Routine Whether To Kick Start The Port By Setting TI
;	When Writing A Character.
;
SP1_INT 	proc
		jb	ri1,SP1_IIN		; If Set, Receiver Interrupt
		jb	ti1,SP1_IOUT		; If Set, Output Interrupt
		clr	B_SP1OFL		; If Weird Interrupt, Clear
		reti				; Neither?  (Should Never Happen!)
		endproc
;
;****************************************************************************
;
;  Description:
;	Process Interrupt Generated By Receiving A Character
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	None
;
;  Affected:
;	None
;
;  Stack:
;	4 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
SP1_IIN 	proc
		push	psw			; Save PSW
		push	acc			; Save Acc
		push	0			; Save R0
;
		mov	a,D_SP1ICC		; Get Input Character Count
		inc	a			; Add One
		cjne	a,#E_SP1IBL,l?p1	; If Not Same, Buffer Not Full
;
		sjmp	l?p3			; Return From Interrupt
;
l?p1		mov	a,D_SP1IBH		; Get End Of Buffer
		push	acc			; Save For Compare
		clr	c			; Clear Carry For Subtract
		subb	a,#E_SP1IBL		; Subtract Input Buffer Length
		pop	acc			; Recover
		jc	l?p2			; If Carry Set, Value Is In Range
		clr	a			; Point Back To Start Of Buffer
l?p2		mov	r0,a			; Store To R0
		inc	a			; Point To Next Header Location
		mov	D_SP1IBH,a		; Store Back
		mov	a,#I_SP1IBB		; Buffer Base Address
		add	a,r0			; Add In Count
		mov	r0,a			; Store Back To R0
		mov	a,sbuf1			; Get Character At Serial Port
		mov	@r0,a			; Store Into Input Buffer
		inc	D_SP1ICC		; Increment Number Of Characters In Buffer
;
l?p3		pop	0			; Recover R0
		pop	acc			; Recover Acc
		pop	psw			; Recover PSW
		clr	ri1			; Acknowledge We Got Character
		reti				; Return To Interrupted Routine
		endproc
;
;****************************************************************************
;
;  Description:
;	Process Interrupt Generated By Empty Transmitter
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	None
;
;  Affected:
;	None
;
;  Stack:
;	4 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
SP1_IOUT	proc
		push	psw			; Save PSW
		push	acc			; Save Acc
		push	0			; Save R0
;
		mov	a,D_SP1OCC		; Get Character Count
		jnz	l?p1			; If Character Count ~0, Stuff To Be Sent
		clr	B_SP1OFL		; Set Not Transmitting
		sjmp	l?p3			; Mark Interrupt As Done, And Return
;
l?p1		mov	a,D_SP1OBH		; Get End Of Buffer
		push	acc			; Save For Compare
		clr	c			; Clear Carry For Subtract
		subb	a,#E_SP1OBL		; Subtract Output Buffer Length
		pop	acc			; Recover
		jc	l?p2			; If Carry Set, Value Is In Range
		clr	a			; Point Back To Start Of Buffer
l?p2		mov	r0,a			; Store To R0
		inc	a			; Point To Next Header Location
		mov	D_SP1OBH,a		; Store Back
		mov	a,#I_SP1OBB		; Output Buffer Base Address
		add	a,r0			; Add In Count
		mov	r0,a			; Store Back To R0
		mov	a,@r0			; Fetch Character From Output Buffer
		mov	sbuf1,a			; Get Character At Serial Port
		dec	D_SP1OCC		; Decrement Number Of Characters In Buffer
;
l?p3		pop	0			; Recover R0
		pop	acc			; Recover Acc
		pop	psw			; Recover PSW
		clr	ti1			; Transmitter Interrupt Acknowledged
		reti				; Return To Interrupted Routine
		endproc
;
;****************************************************************************
;
;  Description:
;	Get A Character From The Serial Buffer, Waiting If None Present
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	Acc Contains Received Character
;
;  Affected:
;	PSW.CY, PSW.Z, PSW.P, Acc
;
;  Stack:
;	2 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	If Serial Interrupts Are Disabled (ES1 == 0), And A Character Is 
;	Present In The Buffer, Then The Serial Interrupt Will Be ReEnabled
;	On Exit.  If ES1 == 0 And No Characters Are In The Buffer, Then
;	The Routine Will Hang Waiting For A Serial Interrupt To Place A
;	Character Into The Buffer.
;
SP1_IN		proc
		call	SP1_INS 		; See If Character Pending
		jnc	SP1_IN			; Nope, Wait For One To Show Up
;
		push	0			; Save R0
		clr	es1			; Disable Serial Interrupts
		mov	a,D_SP1IBT		; Get Tail
		push	acc			; Save Acc
		clr	c			; Clear Carry For Subtract
		subb	a,#E_SP1IBL		; Subtract Length
		pop	acc			; Recover Acc
		jc	l?p1			; If Carry, We Are In SP1IBL Length
		clr	a			; Point To Start of Buffer
l?p1		mov	r0,a			; Store In R0
		inc	a			; Point To Next Location
		mov	D_SP1IBT,a		; Store Updated Tail Pointer
		mov	a,#I_SP1IBB		; Input Buffer Base Address
		add	a,r0			; Add In Count
		mov	r0,a			; Store Back To R0
		mov	a,@r0			; Fetch Character From Output Buffer
		dec	D_SP1ICC		; Decrement Number Of Characters In Buffer
		setb	es1			; Re-Enable Serial Interrupts
		pop	0			; Recover R0
		ret				; Return It To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Return Carry Clear If No Character Pending, Else Carry Set
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	CY == 1 If Character Present, CY == 0 If None
;
;  Affected:
;	PSW.CY
;
;  Stack:
;	1 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
SP1_INS 	proc
		push	acc			; Save Acc
		mov	a,D_SP1IBH		; Get Input Header
		cjne	a,D_SP1IBT,l?p1		; If Not Same, Character Pending
		clr	c			; Clear Carry, Because No Character Is Here
		pop	acc			; Recover Acc
		ret				; Return To Caller
;
l?p1		setb	c			; Set Carry Flag, Character Is Here
		pop	acc			; Recover Acc
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Clears Any Character In The Receive Character Buffer
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	Acc = 0
;
;  Affected:
;	PSW.Z, PSW.P, Acc
;
;  Stack:
;	0 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	Probably Would Be A Good Idea To Disable Receive Interrupts While
;	Doing This.  An Interrupt In The Middle May Cause Problems.
;
SP1_INCLEAR	proc
		clr	a			; 0
		mov	D_SP1IBH,a		; Set Input Buffer Header To 0
		mov	D_SP1IBT,a		; Set Input Buffer Tail To 0
		mov	D_SP1ICC,a		; Set Input Character Count To 0
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Output Character To Serial Buffer
;
;  Entry Requirements:
;	Acc Has Character To Write To Output Buffer
;
;  On Exit:
;	None
;
;  Affected:
;	PSW.CY
;
;  Stack:
;	3 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	If The Output Buffer Is Full, We Spin Waiting For A Serial Transmit
;	Interrupt To Free Up A Byte.  If Serial Interrupts Are Disabled
;	(ES1 == 0) When This Routine Is Called And There Is At Least One Byte
;	Avilable In The Buffer, Interrupts Will Be Reenabled.  If The Buffer 
;	Is Full, The Routine Will Hang Waiting For Space To Become Available
;	In The Buffer.
;
SP1_OUT 	proc
		push	0			; Save R0
		push	acc			; Save Acc
;		
l?p1		mov	a,D_SP1OCC		; Get Output Character Count
		inc	a			; Add 1
		cjne	a,#E_SP1OBL,l?p2	; If Not Same, Buffer Not Full
		sjmp	l?p1			; If Same, Wait Until Buffer Empties
;
l?p2		clr	es1			; Disable Interrupts
;
		mov	a,D_SP1OBT		; Get End Of Buffer
		push	acc			; Save For Compare
		clr	c			; Clear Carry For Subtract
		subb	a,#E_SP1OBL		; Subtract Output Buffer Length
		pop	acc			; Recover
		jc	l?p3			; If Carry Set, Value Is In Range
		clr	a			; Point Back To Start Of Buffer
l?p3		mov	r0,a			; Store In R0
		inc	a			; Next Buffer Location
		mov	D_SP1OBT,a		; Store Back
		mov	a,#I_SP1OBB		; Output Buffer Base Address
		add	a,r0			; Add In Count
		mov	r0,a			; Store Back To R0
		pop	acc			; Get Character To Put In Buffer
		push	acc			; Resave Character
		mov	@r0,a			; Store Character To Output Buffer
		inc	D_SP1OCC		; Increment Number Of Characters In Buffer
;
		jb	B_SP1OFL,l?p4		; If Output In Progress, Skip
		setb	B_SP1OFL		; Otherwise, Set Output In Progress Flag
		setb	ti1			; Force Transmitter Interrupt
l?p4		setb	es1			; Enable Interrupts
		pop	acc			; Recover Acc
		pop	0			; Recover R0
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Return Carry Clear If Output Buffer Is Not Full, Else Carry Set
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	CY == 1 If Output Buffer Full, CY == 0 If Not
;
;  Affected:
;	PSW.CY, PSW.Z, PSW.P, Acc
;
;  Stack:
;	0 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
SP1_OUTS 	proc
		mov	a,D_SP1OCC		; Get Output Character Count
		inc	a			; Add 1
		cjne	a,#E_SP1OBL,l?p1	; If Not Same, Buffer Not Full
		setb	c			; CY == 1 Means Buffer Full
		ret				; Return To Caller
;
l?p1		clr	c			; CY == 0 Means Not Full
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Return CY == 1 If Output Buffer Empty, Else CY == 0
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	CY == 1 If Output Buffer Empty, CY == 0 If Not
;
;  Affected:
;	PSW.CY, PSW.Z, PSW.P, Acc
;
;  Stack:
;	0 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
SP1_OUTEMPTY 	proc
		mov	a,D_SP1OCC		; Get Output Character Count
		jnz	l?p1			; If Not Empty
		setb	c			; CY ==	1 Means	Buffer Empty
		ret				; Return To Caller
;
l?p1		clr	c			; CY ==	0 Means	Not Empty
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Data Area
;
		seg	bit
B_SP1OFL	ds	1			; Serial Output Flag
;
		seg	data
D_SP1IBH	ds	1			; Serial Input Head Pointer
D_SP1IBT	ds	1			; Serial Input Tail Pointer
D_SP1ICC	ds	1			; Serial Input Character Count
D_SP1OBH	ds	1			; Serial Output Buffer Head Pointer
D_SP1OBT	ds	1			; Serial Output Buffer Tail Pointer
D_SP1OCC	ds	1			; Serial Output Character Count

		seg	idata
I_SP1IBB	ds	E_SP1IBL		; Serial Input Buffer
I_SP1OBB	ds	E_SP1OBL		; Serial Output Buffer
;
;*******************************************************************************
;
		end
